#!/usr/bin/env python3
import os
import json
from typing import Any
from datetime import datetime
from pathlib import Path

import pymysql
from apscheduler.schedulers.blocking import BlockingScheduler
from pymysql.cursors import DictCursor

WP_POSTVIEWS_GET_COUNTS_SQL = """select p.ID, p.post_title, cast(m.meta_value as signed integer)
as view_cnt, p.guid as url, p.post_type
from wp_postmeta m, wp_posts p
where m.meta_key = 'views'
      and m.post_id = p.ID
order by view_cnt desc;
"""


def connect() -> pymysql.Connection:
    host, port_str = os.environ["DB_HOST"].split(":")

    return pymysql.connect(
        host=host,
        user=os.environ["DB_USER"],
        password=os.environ["DB_PASSWORD"],
        database=os.environ["DB_NAME"],
        port=int(port_str),
        cursorclass=DictCursor,
    )


def get_views() -> list[dict[str, Any]]:
    with connect() as connection:
        with connection.cursor() as cursor:
            cursor.execute(WP_POSTVIEWS_GET_COUNTS_SQL)
            return cursor.fetchall()


def get_and_save_views() -> None:
    now = datetime.now()
    now_str = now.isoformat().split(".")[0].replace(":", "_") + "Z"
    views = get_views()

    base_dir = Path("./instance/wp_postviews")
    base_dir.mkdir(exist_ok=True, parents=True)

    with open(base_dir / f"{now_str}.json", "w") as f:
        json.dump(views, f, ensure_ascii=False, indent=4)


if __name__ == "__main__":
    scheduler = BlockingScheduler()
    scheduler.add_job(get_and_save_views, "cron", hour=0, minute=0)
    scheduler.start()
